==> mui_data_grid-0.5.0.tar.gz: mui_data_grid-0.5.0/src/mui/v5/integrations/sqlalchemy/structures/query.py <==
"""The query module contains the DataGridQuery data structure.

This structure is used to provide helpers related to pagination, such as
total row counts.
"""
from typing import Generic, List, Optional, TypeVar

from sqlalchemy.orm import Query

from mui.v5.grid import GridFilterModel, GridPaginationModel, GridSortModel
from mui.v5.integrations.sqlalchemy.filter import apply_filter_to_query_from_model
from mui.v5.integrations.sqlalchemy.pagination import (
    apply_limit_offset_to_query_from_model,
)
from mui.v5.integrations.sqlalchemy.resolver import Resolver
from mui.v5.integrations.sqlalchemy.sort import apply_sort_to_query_from_model

_T = TypeVar("_T")


class DataGridQuery(Generic[_T]):
    """A data grid query handles utilities related to our query.

    Args:
        Generic (_type_): The model being retrieved by the query.
    """

    _query: "Query[_T]"
    column_resovler: Resolver
    filter_model: Optional[GridFilterModel]
    pagination_model: Optional[GridPaginationModel]
    query: "Query[_T]"
    sort_model: Optional[GridSortModel]

    def __init__(
        self,
        query: "Query[_T]",
        column_resolver: Resolver,
        filter_model: Optional[GridFilterModel] = None,
        sort_model: Optional[GridSortModel] = None,
        pagination_model: Optional[GridPaginationModel] = None,
    ) -> None:
        """Initialize a new data grid query.

        Args:
            query (Query[_T]): The base query which the models will be applied to.
            column_resolver (Resolver): The field resolver which converts a UI field
                to the corresponding SQLAlchemy column, column property, etc.
            filter_model (Optional[GridFilterModel], optional): The filter model to
                apply, if provided. Defaults to None.
            sort_model (Optional[GridSortModel], optional): The sort model to apply,
                if provided. Defaults to None.
            pagination_model (Optional[GridPaginationModel], optional): The pagination
                model to apply, if provided. Defaults to None.
        """
        self.column_resovler = column_resolver
        self.filter_model = filter_model
        self.sort_model = sort_model
        self.pagination_model = pagination_model
        query = self._filter_query(query=query)
        # we filter it first, so that our total is accurate
        self._query = query
        # then we apply the order and pagination limits
        query = self._order_query(query=query)
        query = self._paginate_query(query=query)
        self.query = query

    def _filter_query(self, query: "Query[_T]") -> "Query[_T]":
        """Applies the filter model to the query.

        Args:
            query (Query[_T]): The query being filtered.

        Returns:
            Query[_T]: The filtered query.
        """
        if self.filter_model is None:
            return query
        return apply_filter_to_query_from_model(
            query=query, model=self.filter_model, resolver=self.column_resovler
        )

    def _order_query(self, query: "Query[_T]") -> "Query[_T]":
        """Applies the sort model to the query.

        Args:
            query (Query[_T]): The query being ordered / sorted.

        Returns:
            Query[_T]: The sorted / ordered query.
        """
        if self.sort_model is None:
            return query
        return apply_sort_to_query_from_model(
            query=query, model=self.sort_model, resolver=self.column_resovler
        )

    def _paginate_query(self, query: "Query[_T]") -> "Query[_T]":
        """Applies the pagination model to the query.

        Args:
            query (Query[_T]): The query being paginated (limit / offset).

        Returns:
            Query[_T]: The paginated (limited) query.
        """
        if self.pagination_model is None:
            return query
        return apply_limit_offset_to_query_from_model(
            query=query, model=self.pagination_model
        )

    def total(self) -> int:
        """Returns the total number of rows that exist with the filter.

        Returns:
            int: _description_
        """
        return self._query.order_by(None).count()

    def items(self) -> List[_T]:
        """Returns the results of the query.

        Returns:
            List[_T]: The list of individual items located by the query after all
                models have been applied.
        """
        return self.query.all()

==> mui_data_grid-0.6.0.tar.gz: mui_data_grid-0.6.0/src/mui/v5/integrations/sqlalchemy/structures/query.py <==
"""The query module contains the DataGridQuery data structure.

This structure is used to provide helpers related to pagination, such as
total row counts.
"""
from math import ceil
from typing import Callable, Generic, List, Optional, TypeVar, Union, overload

from sqlalchemy.orm import Query

from mui.v5.grid import GridFilterModel, GridPaginationModel, GridSortModel
from mui.v5.integrations.sqlalchemy.filter import apply_filter_to_query_from_model
from mui.v5.integrations.sqlalchemy.pagination import (
    apply_limit_offset_to_query_from_model,
)
from mui.v5.integrations.sqlalchemy.resolver import Resolver
from mui.v5.integrations.sqlalchemy.sort import apply_sort_to_query_from_model

_T = TypeVar("_T")
_R = TypeVar("_R")


class DataGridQuery(Generic[_T]):
    """A data grid query handles utilities related to our query.

    Args:
        Generic (_type_): The model being retrieved by the query.
    """

    _query: "Query[_T]"
    column_resovler: Resolver
    filter_model: Optional[GridFilterModel]
    pagination_model: Optional[GridPaginationModel]
    query: "Query[_T]"
    sort_model: Optional[GridSortModel]

    def __init__(
        self,
        query: "Query[_T]",
        column_resolver: Resolver,
        filter_model: Optional[GridFilterModel] = None,
        sort_model: Optional[GridSortModel] = None,
        pagination_model: Optional[GridPaginationModel] = None,
    ) -> None:
        """Initialize a new data grid query.

        Args:
            query (Query[_T]): The base query which the models will be applied to.
            column_resolver (Resolver): The field resolver which converts a UI field
                to the corresponding SQLAlchemy column, column property, etc.
            filter_model (Optional[GridFilterModel], optional): The filter model to
                apply, if provided. Defaults to None.
            sort_model (Optional[GridSortModel], optional): The sort model to apply,
                if provided. Defaults to None.
            pagination_model (Optional[GridPaginationModel], optional): The pagination
                model to apply, if provided. Defaults to None.
        """
        self.column_resovler = column_resolver
        self.filter_model = filter_model
        self.sort_model = sort_model
        self.pagination_model = pagination_model
        query = self._filter_query(query=query)
        # we filter it first, so that our total is accurate
        self._query = query
        # then we apply the order and pagination limits
        query = self._order_query(query=query)
        query = self._paginate_query(query=query)
        self.query = query

    def _filter_query(self, query: "Query[_T]") -> "Query[_T]":
        """Applies the filter model to the query.

        Args:
            query (Query[_T]): The query being filtered.

        Returns:
            Query[_T]: The filtered query.
        """
        if self.filter_model is None:
            return query
        return apply_filter_to_query_from_model(
            query=query, model=self.filter_model, resolver=self.column_resovler
        )

    def _order_query(self, query: "Query[_T]") -> "Query[_T]":
        """Applies the sort model to the query.

        Args:
            query (Query[_T]): The query being ordered / sorted.

        Returns:
            Query[_T]: The sorted / ordered query.
        """
        if self.sort_model is None:
            return query
        return apply_sort_to_query_from_model(
            query=query, model=self.sort_model, resolver=self.column_resovler
        )

    def _paginate_query(self, query: "Query[_T]") -> "Query[_T]":
        """Applies the pagination model to the query.

        Args:
            query (Query[_T]): The query being paginated (limit / offset).

        Returns:
            Query[_T]: The paginated (limited) query.
        """
        if self.pagination_model is None:
            return query
        return apply_limit_offset_to_query_from_model(
            query=query, model=self.pagination_model
        )

    def total(self) -> int:
        """Returns the total number of rows that exist with the filter.

        Returns:
            int: _description_
        """
        return self._query.order_by(None).count()

    @property
    def per_page(self) -> int:
        """Returns the page size.

        Returns:
            int: 0 if no pagination model exists, otherwise the page size.
        """
        if self.pagination_model is None:
            return 0
        return self.pagination_model.page_size

    @overload
    def items(self, factory: None = ...) -> List[_T]:
        """When a factory function is not provided, simply return the models.

        Args:
            factory (None, optional): This is not provided. Defaults to None.

        Returns:
            List[_T]: The list of models, without conversion.
        """
        ...

    @overload
    def items(self, factory: Callable[[_T], _R]) -> List[_R]:
        """When a factory function is provided, return a list of items created by
        the factory.

        Args:
            factory (Callable[[_T], _R]): The factory to convert the type(s).

        Returns:
            List[_R]: The list of created items.
        """
        ...

    def items(
        self, factory: Optional[Callable[[_T], _R]] = None
    ) -> Union[List[_T], List[_R]]:
        """Returns all results of the query, after all models have been applied.

        Args:
            factory (Optional[Callable[[_T], _R]]): The factory function to convert the
                model into a different type.

        Returns:
            List[_T]: The list of individual items located by the query after all
                models have been applied.
        """
        items = self.query.all()
        if factory is not None:
            return [factory(item) for item in items]
        return items

    def pages(self, total: Optional[int] = None) -> int:
        """Returns the number of pages to display all results.

        Args:
            total (Optional[int], optional): The total number of results. This may
                be provided to avoid the overhead of an additional database query to
                retrieve the total. Defaults to None.

        Returns:
            int: The number of pages required to display all results at the current
                page size.
        """
        if not total:
            total = self.total()
        return int(ceil(total / float(self.per_page)))

    @property
    def page(self) -> int:
        """Returns the current page number.

        Returns:
            int: 0 if no pagination model exists, otherwise the page number.
        """
        if self.pagination_model is None:
            return 0
        return self.pagination_model.page
